#
# This is a specialized test case for orchestrating against furnish-ssh
# with a vagrant box.
#
class Furnish::SSHTestCase < Furnish::SchedulerTestCase
  BOX_URL = "http://files.vagrantup.com/precise64.box"
  PRIVATE_KEY_PATH = "test/data/vagrant" # path to private keys that work on vagrant

  def setup
    super
    @machine_count = 1
    @ip = nil
    @klass = Furnish::Provisioner::SSH
    sched.serial = true
  end

  def teardown
    sched.teardown
    super
  end

  def ssh_provisioner(extra_args={ :startup_command => "ls", :log_output => true })
    args = { :username => "vagrant", :password => "vagrant" }
    @klass.new(args.merge(extra_args))
  end

  def ip
    @ip ||= Furnish::IP.new("10.10.10.0/24")
    @ip.allocate("10.10.10.0")
    @ip.allocate("10.10.10.1")
    @ip
  end

  def machine_provisioner(num=@machine_count)
    [
      Furnish::Provisioner::AutoIP.new(:ip => ip, :number_of_addresses => num),
      Furnish::Provisioner::Vagrant.new(:box_url => BOX_URL, :number_of_servers => num)
    ]
  end
end
