require 'helper'

class TestAPI < Furnish::SSHTestCase
  def test_protocol
    sp = @klass.startup_protocol
    assert_nil(sp[:accepts_from_any])
    assert_includes(sp[:requires], :ips)
    assert_equal(Set, sp[:requires][:ips][:type])
    assert_includes(sp[:yields], :ssh_exit_statuses)
    assert_equal(Hash, sp[:yields][:ssh_exit_statuses][:type])

    sp = @klass.shutdown_protocol
    assert_includes(sp[:yields], :ssh_exit_statuses)
    assert_equal(Hash, sp[:yields][:ssh_exit_statuses][:type])
    assert(sp[:accepts_from_any])
  end

  def test_properties
    fp = @klass.furnish_properties
    {
      :username         => String,
      :password         => String,
      :private_key      => String,
      :private_key_path => String,
      :startup_command  => String,
      :shutdown_command => String,
      :provision_wait   => Numeric
    }.each do |key, value|
      assert_includes(fp, key)
      assert_equal(value, fp[key][:type])
    end
  end

  def test_constructor
    passing_auth_opts = { :username => "foo", :password => "quux" }
    passing_cmd_opts  = { :startup_command => "ls" }

    assert_raises(ArgumentError) { @klass.new }
    assert_raises(ArgumentError) { @klass.new(:username => "foo") }
    assert_raises(ArgumentError) { @klass.new(:password => "bar") }

    # XXX permutations of auth options
    assert_raises(ArgumentError) { @klass.new(passing_cmd_opts.merge(:username => "foo", :password => "quux", :private_key_path => "frob")) }
    assert_raises(ArgumentError) { @klass.new(passing_cmd_opts.merge(:username => "foo", :password => "quux", :private_key => "frob")) }
    assert_raises(ArgumentError) { @klass.new(passing_cmd_opts.merge(:username => "foo", :private_key_path => "quux", :private_key => "frob")) }
    assert_raises(ArgumentError) { @klass.new(passing_cmd_opts.merge(:username => "foo", :private_key_path => "quux", :private_key => "frob", :password => "heyooo")) }

    # same as above, just with valid arguments
    %w[password private_key private_key_path].each do |arg|
      @klass.new(passing_cmd_opts.merge(:username => "foo", arg.to_sym => "quux"))
    end

    assert_raises(ArgumentError) { @klass.new(passing_auth_opts) }

    cmd_opts = %w[startup_command shutdown_command]

    cmd_opts.each do |arg|
      @klass.new(passing_auth_opts.merge(arg.to_sym => "ls"))
    end

    @klass.new(passing_auth_opts.merge(Hash[cmd_opts.map { |arg| [arg.to_sym, "ls"] }]))

    obj = @klass.new(passing_auth_opts.merge(passing_cmd_opts))
    assert_equal(300, obj.provision_wait)

    obj = @klass.new(passing_auth_opts.merge(passing_cmd_opts.merge(:provision_wait => 10)))
    assert_equal(10, obj.provision_wait)
  end
end
