require 'helper'

class TestBasic < Furnish::SSHTestCase
  def test_startup
    sched.s("test1-startup", machine_provisioner)
    sched.run

    prov = ssh_provisioner
    assert_equal(["startup: 'ls'"], prov.report)

    target_ips = ip.group_ips("test1-startup")
    assert_equal(@machine_count, target_ips.count)

    ret = prov.startup(:ips => target_ips)
    refute_nil(ret)
    assert_equal(Hash[target_ips.map { |ip| [ip, 0] }], ret[:ssh_exit_statuses])

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      refute_empty(ret[:ssh_output][ip])
      refute_empty(prov.output[ip])
    end

    ret = prov.shutdown
    refute_nil(ret)
    assert_equal(Hash[target_ips.map { |ip| [ip, 0] }], ret[:ssh_exit_statuses])

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      assert_empty(ret[:ssh_output][ip])
      assert_empty(prov.output[ip])
    end

    prov = ssh_provisioner(:startup_command => 'exit 1')
    refute(prov.startup(:ips => target_ips))
    assert(prov.shutdown)

    prov = ssh_provisioner(:shutdown_command => 'exit 1')
    assert(prov.startup(:ips => target_ips))
    refute(prov.shutdown)

    prov = ssh_provisioner(:startup_command => 'exit 1', :success => Set[0,1])
    assert(prov.startup(:ips => target_ips))
  end

  def test_shutdown
    sched.s("test1-shutdown", machine_provisioner)
    sched.run

    prov = ssh_provisioner(:shutdown_command => "ls")
    assert_equal(["shutdown: 'ls'"], prov.report)

    refute(prov.shutdown)

    target_ips = ip.group_ips("test1-shutdown")
    assert_equal(@machine_count, target_ips.count)

    ret = prov.startup(:ips => Set[])
    refute_nil(ret)
    assert_empty(ret[:ssh_exit_statuses])
    assert_empty(ret[:ssh_output])
    assert_empty(prov.output)

    refute(prov.shutdown(:ips => Set[]))
    refute(prov.shutdown)

    ret = prov.shutdown(:ips => target_ips)
    refute_nil(ret)
    assert_equal(Hash[target_ips.map { |ip| [ip, 0] }], ret[:ssh_exit_statuses])

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      refute_empty(ret[:ssh_output][ip])
      refute_empty(prov.output[ip])
    end

    prov = ssh_provisioner(:shutdown_command => 'exit 1')
    assert(prov.startup(:ips => Set[]))
    refute(prov.shutdown(:ips => target_ips))

    prov = ssh_provisioner(:shutdown_command => 'exit 1', :success => Set[0,1])
    assert(prov.startup(:ips => target_ips))
    assert(prov.shutdown)
  end

  def test_basic_provision
    sched.s("test1-basic", machine_provisioner + [ssh_provisioner, ssh_provisioner(:shutdown_command => "ls")])
    # these next two will raise if something's not working.
    sched.run
    sched.down("test1-basic")
  end

  def test_failing_provision
    sched.s("test1-failing", machine_provisioner + [ssh_provisioner(:startup_command => 'exit 1'), ssh_provisioner(:shutdown_command => "ls")])
    assert_raises(RuntimeError) { sched.run }
    sched.force_deprovision = true
    sched.down("test1-failing")
    sched.force_deprovision = false
    sched.s("test2-failing", machine_provisioner + [ssh_provisioner, ssh_provisioner(:shutdown_command => "exit 1")])
    sched.run
    assert_raises(RuntimeError) { sched.down("test2-failing") }
    sched.force_deprovision = true
    sched.down("test2-failing")
  ensure
    sched.force_deprovision = false
  end
end
