require 'helper'

class TestFeatures < Furnish::SSHTestCase
  def test_stdin_and_log_and_merge_output
    if ENV["FURNISH_DEBUG"]
      $stderr.puts "Log tests are running"
    end

    io = StringIO.new('', 'w')

    Furnish.logger = Furnish::Logger.new(io, 3)

    sched.s("test1-stdin", machine_provisioner)
    sched.run

    prov = ssh_provisioner(:startup_command => "cat", :stdin => "fart\n", :log_output => true)
    target_ips = ip.group_ips("test1-stdin")
    ret = prov.startup(:ips => target_ips)
    assert(ret)

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      assert_equal("fart\n", ret[:ssh_output][ip])
      assert_equal("fart\n", prov.output[ip])
      assert_match(/^\[#{Regexp.quote(ip)}\] fart$/, io.string)
    end

    io.string = ''

    prov = ssh_provisioner(:startup_command => "cat >/dev/fd/2", :stdin => "fart\n", :log_output => true)
    target_ips = ip.group_ips("test1-stdin")
    ret = prov.startup(:ips => target_ips)
    assert(ret)

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      assert_empty(ret[:ssh_output][ip])
      assert_empty(prov.output[ip])
      assert_empty(io.string)
    end

    prov = ssh_provisioner(:startup_command => "cat >/dev/fd/2", :stdin => "fart\n", :merge_output => true, :log_output => true)
    target_ips = ip.group_ips("test1-stdin")
    ret = prov.startup(:ips => target_ips)
    assert(ret)

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      assert_equal("fart\n", ret[:ssh_output][ip])
      assert_equal("fart\n", prov.output[ip])
      assert_match(/^\[#{Regexp.quote(ip)}\] fart$/, io.string)
    end

    prov = ssh_provisioner(:startup_command => "cat >/dev/fd/2", :stdin => "fart\n", :merge_output => true, :log_output => true, :mute_output => true)
    target_ips = ip.group_ips("test1-stdin")
    ret = prov.startup(:ips => target_ips)
    assert(ret)

    target_ips.each do |ip|
      assert_kind_of(String, ret[:ssh_output][ip])
      assert_empty(ret[:ssh_output][ip])
      assert_empty(prov.output[ip])
      assert_match(/^\[#{Regexp.quote(ip)}\] fart$/, io.string)
    end
  end
end
